from fishy.systems.fishing_event import HookEvent, StickEvent, LookEvent, IdleEvent
from fishy.systems.fishing_mode import FishingMode
from fishy.systems.globals import G
from fishy.systems.pixel_loc import PixelLoc
import fishy.systems.fishy_network as net
from fishy.systems.window import Window
from fishy.systems.auto_update import auto_upgrade



import logging
import os
import shutil
import sys
import threading
import traceback
import webbrowser
from decimal import Decimal
from threading import Thread
from zipfile import ZipFile

import cv2
import numpy as np
from uuid import uuid1
from hashlib import md5

import requests
from whatsmyip.ip import get_ip
from whatsmyip.providers import GoogleDnsProvider

import fishy
from fishy.systems.config import Config
import functools

from fishy.systems.gui import GUIFunction


def round_float(v, ndigits=2, rt_str=False):
    """
    Rounds float
    :param v: float ot round off
    :param ndigits: round off to ndigits decimal points
    :param rt_str: true to return string
    :return: rounded float or strings
    """
    d = Decimal(v)
    v_str = ("{0:.%sf}" % ndigits).format(round(d, ndigits))
    if rt_str:
        return v_str
    return Decimal(v_str)


def draw_keypoints(vis, keypoints, color=(0, 0, 255)):
    """
    draws a point on cv2 image array
    :param vis: cv2 image array to draw
    :param keypoints: keypoints array to draw
    :param color: color of the point
    """
    for kp in keypoints:
        x, y = kp.pt
        cv2.circle(vis, (int(x), int(y)), 5, color, -1)


def enable_full_array_printing():
    """
    Used to enable full array logging
    (summarized arrays are printed by default)
    """
    np.set_printoptions(threshold=sys.maxsize)


def open_web(website):
    logging.debug("opening web, please wait...")
    Thread(target=lambda: webbrowser.open(website, new=2)).start()


def disable_logging(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        logging.disable(logging.DEBUG)
        result = func(*args, **kwargs)
        logging.disable(logging.NOTSET)
        return result

    return wrapper


@disable_logging
def req(config: Config, data):
    url = 'https://dcserver1.herokuapp.com/fishy'
    ip = get_ip(GoogleDnsProvider)
    h = config.get("hash", md5(str(uuid1()).encode()).hexdigest())
    body = {"hash": h, "data": data, "ip": ip}
    requests.post(url, json=body)


def ping(config: Config, data):
    if config.get("check", True, False):
        threading.Thread(target=req, args=(config, data,)).start()


def install_thread_excepthook():
    """
    Workaround for sys.excepthook thread bug
    https://bugs.python.org/issue1230540
    (https://sourceforge.net/tracker/?func=detail&atid=105470&aid=1230540&group_id=5470).
    Call once from __main__ before creating any threads.
    If using psyco, call psycho.cannotcompile(threading.Thread.run)
    since this replaces a new-style class method.
    """
    import sys
    run_old = threading.Thread.run

    def run(*args, **kwargs):
        try:
            run_old(*args, **kwargs)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            sys.excepthook(*sys.exc_info())

    threading.Thread.run = run


def unhandled_exception_logging(*exc_info):
    text = "".join(traceback.format_exception(*exc_info))
    logging.error("Unhandled exception: %s", text)


def get_data_file_path(rel_path):
    return os.path.join(os.path.dirname(fishy.__file__), rel_path)


def create_shortcut(gui):
    try:
        user = os.path.expanduser("~")
        if os.path.exists(os.path.join(user, "Desktop")):
            path = os.path.join(user, "Desktop", "Fishybot ESO.lnk")
            _copy_shortcut(path)
        else:
            gui.call(GUIFunction.ASK_DIRECTORY, (_copy_shortcut,
                                                 "Could not find Desktop please specify path to create shortcut"))
    except Exception:
        logging.info("Couldn't create shortcut")
        traceback.print_exc()


def _copy_shortcut(path):
    shutil.copy(get_data_file_path('FishybotESO.lnk'), path)
    logging.info("Shortcut created")


def check_addon():
    try:
        user = os.path.expanduser("~")
        addon_dir = os.path.join(user, "Documents", "Elder Scrolls Online", "live", "Addons")
        if not os.path.exists(os.path.join(addon_dir, 'ProvisionsChalutier')):
            logging.info("Addon not found, installing it...")
            with ZipFile(get_data_file_path("ProvisionsChalutier.zip"), 'r') as zip:
                zip.extractall(path=addon_dir)
            logging.info("Please make sure you enable \"Allow outdated addons\" in-game\n"
                         "Also, make sure the addon is visible clearly on top left corner of the game window")
    except Exception:
        print("couldn't install addon, try doing it manually")



import logging
import socket
import json

PORT = 8023
MESSAGE = "yo"
RETRY_LIMIT = 5
IP = 0
s = None


def initialize(ip):
    global s, IP
    IP = ip


def send_message(message, count=1):
    try:
        sock = socket.socket()
        sock.connect((IP, PORT))
        sock.send(bytes(message, "utf-8"))
        sock.close()
    except ConnectionRefusedError:
        logging.info("Connection Refused, please turn on service on mobile")
    except TimeoutError:
        logging.info("Timeout Error")

        if count < RETRY_LIMIT:
            send_message(message, count+1)
        

def sendHoleDeplete(count):
    message = {"action": "holeDeplete", "fishCount": count}
    jsonString = json.dumps(message)
    send_message(jsonString)


if __name__ == "__main__":
    initialize("192.168.1.102")
    sendHoleDeplete(2)



import logging
import os
import time
from enum import Enum
from logging import StreamHandler
from tkinter import *
from tkinter import filedialog, messagebox
from tkinter.ttk import *
from typing import Tuple, List, Callable, Optional

from ttkthemes import ThemedTk
import threading

from fishy.systems import helper
from fishy.systems.config import Config


class GUIStreamHandler(StreamHandler):
    def __init__(self, gui):
        StreamHandler.__init__(self)
        self.gui = gui

    def emit(self, record):
        msg = self.format(record)
        self.gui.call(GUIFunction.LOG, (msg,))


class GUIEvent(Enum):
    START_BUTTON = 0  # args: ip: str, action_key: str, fullscreen: bool, collect_r: bool
    CHECK_PIXELVAL = 1
    QUIT = 2


class GUIFunction(Enum):
    LOG = 0  # args: str
    STARTED = 1  # args: bool
    ASK_DIRECTORY = 2  # callback: callable


class GUI:

    def __init__(self, config: Config, event_trigger: Callable[[GUIEvent, Optional[Tuple]], None]):
        self.config = config
        self.start_restart = False
        self.destroyed = True
        self._log_strings = []
        self._function_queue: List[Tuple[GUIFunction, Tuple]] = []
        self._event_trigger = event_trigger
        self._bot_running = False

        # UI items
        self.root = None
        self.console = None
        self.start_button = None

        self.thread = threading.Thread(target=self.create, args=())

        rootLogger = logging.getLogger('')
        rootLogger.setLevel(logging.DEBUG)
        new_console = GUIStreamHandler(self)
        rootLogger.addHandler(new_console)

    def create(self):
        self.root = ThemedTk(theme="equilux", background=True)
        self.root.title("Fiishybot for Elder Scrolls Online")
        self.root.geometry('650x550')

        self.root.iconbitmap(helper.get_data_file_path('icon.ico'))

        # region menu
        menubar = Menu(self.root)

        filemenu = Menu(menubar, tearoff=0)
        filemenu.add_command(label="Create Shortcut", command=lambda: helper.create_shortcut(self))

        dark_mode_var = IntVar()
        dark_mode_var.set(int(self.config.get('dark_mode', True)))
        filemenu.add_checkbutton(label="Dark Mode", command=self._toggle_mode,
                                 variable=dark_mode_var)

        menubar.add_cascade(label="File", menu=filemenu)

        debug_menu = Menu(menubar, tearoff=0)
        debug_menu.add_command(label="Check PixelVal",
                               command=lambda: self._event_trigger(GUIEvent.CHECK_PIXELVAL))

        debug_var = IntVar()
        debug_var.set(int(self.config.get('debug', False)))

        def keep_console():
            self.config.set("debug", bool(debug_var.get()))
            logging.debug("Restart to update the changes")
        debug_menu.add_checkbutton(label="Keep Console", command=keep_console, variable=debug_var)

        debug_menu.add_command(label="Log Dump", command=lambda: logging.error("Not Implemented"))
        menubar.add_cascade(label="Debug", menu=debug_menu)

        help_menu = Menu(menubar, tearoff=0)
        help_menu.add_command(label="Troubleshoot Guide", command=lambda: logging.debug("Not Implemented"))
        help_menu.add_command(label="Need Help?", command=lambda: helper.open_web("http://discord.definex.in"))
        help_menu.add_command(label="Donate", command=lambda: helper.open_web("https://paypal.me/AdamSaudagar"))
        menubar.add_cascade(label="Help", menu=help_menu)

        self.root.config(menu=menubar)
        # endregion

        # region console
        self.console = Text(self.root, state='disabled', wrap='none', background="#707070", fg="#ffffff")
        self.console.pack(fill=BOTH, expand=True, pady=(15, 15), padx=(5, 5))
        self.console.mark_set("sentinel", INSERT)
        self.console.config(state=DISABLED)

        controls_frame = Frame(self.root)
        # endregion

        # region controls
        left_frame = Frame(controls_frame)

        Label(left_frame, text="Android IP").grid(row=0, column=0)
        ip = Entry(left_frame)
        ip.insert(0, self.config.get("ip", ""))
        ip.grid(row=0, column=1)

        Label(left_frame, text="Fullscreen: ").grid(row=1, column=0, pady=(5, 5))
        borderless = Checkbutton(left_frame, variable=IntVar(value=int(self.config.get("borderless", False))))
        borderless.grid(row=1, column=1)

        left_frame.grid(row=0, column=0)

        right_frame = Frame(controls_frame)

        Label(right_frame, text="Action Key:").grid(row=0, column=0)
        action_key_entry = Entry(right_frame)
        action_key_entry.grid(row=0, column=1)
        action_key_entry.insert(0, self.config.get("action_key", "e"))

        Label(right_frame, text="Collect R: ").grid(row=1, column=0, pady=(5, 5))
        collect_r = Checkbutton(right_frame, variable=IntVar(value=1 if self.config.get("collect_r", False) else 0))
        collect_r.grid(row=1, column=1)

        right_frame.grid(row=0, column=1, padx=(50, 0))

        controls_frame.pack()

        self.start_button = Button(self.root, text="STOP" if self._bot_running else "START", width=25)

        def start_button_callback():
            args = (ip.get(),
                    action_key_entry.get(),
                    borderless.instate(['selected']),
                    collect_r.instate(['selected']))
            self._event_trigger(GUIEvent.START_BUTTON, args)
            self._save_config(*args)

        self.start_button["command"] = start_button_callback
        self.start_button.pack(pady=(15, 15))
        # endregion

        self._apply_theme(self.config.get("dark_mode", True))
        self.root.update()
        self.root.minsize(self.root.winfo_width(), self.root.winfo_height())
        self.root.protocol("WM_DELETE_WINDOW", self._set_destroyed)
        self.destroyed = False

        while True:
            self.root.update()
            self._clear_function_queue()
            if self.start_restart:
                self.root.destroy()
                self.root.quit()
                self.start_restart = False
                self.create()
            if self.destroyed:
                self._event_trigger(GUIEvent.QUIT)
                break
            time.sleep(0.01)

    def _clear_function_queue(self):
        while len(self._function_queue) > 0:
            func = self._function_queue.pop(0)

            if func[0] == GUIFunction.LOG:
                self._write_to_console(func[1][0])
            elif func[0] == GUIFunction.STARTED:
                self._bot_running = func[1][0]
                self.start_button["text"] = "STOP" if self._bot_running else "START"
            elif func[0] == GUIFunction.ASK_DIRECTORY:
                messagebox.showinfo("Directory?", func[1][1])
                path = filedialog.askdirectory()
                if path != '':
                    threading.Thread(target=func[1][0], args=(path,)).start()

    def _apply_theme(self, dark):
        self.root["theme"] = "equilux" if dark else "breeze"
        self.console["background"] = "#707070" if dark else "#ffffff"
        self.console["fg"] = "#ffffff" if dark else "#000000"

    def _toggle_mode(self):
        self.config.set("dark_mode", not self.config.get("dark_mode", True))
        self.start_restart = True

    def _set_destroyed(self):
        self.destroyed = True

    def _write_to_console(self, msg):
        numlines = self.console.index('end - 1 line').split('.')[0]
        self.console['state'] = 'normal'
        if int(numlines) >= 50:  # delete old lines
            self.console.delete(1.0, 2.0)
        if self.console.index('end-1c') != '1.0':  # new line for each log
            self.console.insert('end', '\n')
        self.console.insert('end', msg)
        self.console.see("end")  # scroll to bottom
        self.console['state'] = 'disabled'

    def _save_config(self, ip, action_key, borderless, collect_r):
        self.config.set("ip", ip, False)
        self.config.set("action_key", action_key, False)
        self.config.set("borderless", borderless, False)
        self.config.set("collect_r", collect_r, False)
        self.config.save_config()

    def start(self):
        self.thread.start()

    def call(self, gui_func: GUIFunction, args: Tuple = None):
        self._function_queue.append((gui_func, args))



import json
import os
from threading import Thread

filename = os.path.expanduser(r"~/Documents/fishy_config.json")


class Config:

    def __init__(self):
        self.config_dict = json.loads(open(filename).read()) if os.path.exists(filename) else dict()

    def get(self, key, default=None, save=True):
        if key in self.config_dict:
            return self.config_dict[key]

        if save:
            self.set(key, default)
        return default

    def set(self, key, value, save=True):
        self.config_dict[key] = value
        if save:
            self.save_config()

    def save_config(self):
        def save():
            with open(filename, 'w') as f:
                f.write(json.dumps(self.config_dict))

        Thread(target=save).start()



import logging
import sys
import time
import win32con
import win32gui
from threading import Thread

import cv2
import pywintypes
import fishy
from fishy.systems import *
from fishy.systems import helper
from fishy.systems.config import Config
from fishy.systems.gui import GUI, GUIEvent, GUIFunction


class Fishy:
    def __init__(self, gui_ref, gui_event_buffer):
        self.gui_events = gui_event_buffer
        self.start = False
        self.fishPixWindow = None
        self.fishy_thread = None
        self.gui = gui_ref

    def start_fishing(self, ip: str, action_key: str, borderless: bool, collect_r: bool):
        """
        Starts the fishing
        code explained in comments in detail
        """

        if ip != "":
            net.initialize(ip)

        # initialize widow
        try:
            Window.Init(borderless)
        except pywintypes.error:
            logging.info("Game window not found")
            self.start = False
            return

        # initializes fishing modes and their callbacks
        FishingMode("hook", 0, HookEvent(action_key, collect_r))
        FishingMode("stick", 1, StickEvent())
        FishingMode("look", 2, LookEvent())
        FishingMode("idle", 3, IdleEvent(ip != ""))

        self.fishPixWindow = Window(color=cv2.COLOR_RGB2HSV)

        # check for game window and stuff
        self.gui.call(GUIFunction.STARTED, (True,))
        logging.info("Starting the bot engine, look at the fishing hole to start fishing")
        Thread(target=wait_and_check).start()
        while self.start:
            # Services to be ran in the start of the main loop
            Window.Loop()

            # get the PixelLoc and find the color values, to give it to `FishingMode.Loop`
            self.fishPixWindow.crop = PixelLoc.val
            hueValue = self.fishPixWindow.getCapture()[0][0][0]
            FishingMode.Loop(hueValue)
            # Services to be ran in the end of the main loop
            Window.LoopEnd()
        logging.info("Fishing engine stopped")
        self.gui.call(GUIFunction.STARTED, (False,))

    def start_event_handler(self):
        while True:
            while len(self.gui_events) > 0:
                event = self.gui_events.pop(0)

                if event[0] == GUIEvent.START_BUTTON:
                    self.start = not self.start
                    if self.start:
                        self.fishy_thread = Thread(target=self.start_fishing, args=(*event[1],))
                        self.fishy_thread.start()
                elif event[0] == GUIEvent.CHECK_PIXELVAL:
                    if self.start:
                        self.show_pixel_vals()
                    else:
                        logging.debug("Start the engine first before running this command")
                elif event[0] == GUIEvent.QUIT:
                    self.start = False
                    return

    def show_pixel_vals(self):
        def show():
            freq = 0.5
            t = 0
            while t < 10.0:
                t += freq
                logging.debug(str(FishingMode.CurrentMode.label) + ":" + str(self.fishPixWindow.getCapture()[0][0]))
                time.sleep(freq)

        logging.debug("Will display pixel values for 10 seconds")
        time.sleep(5)
        Thread(target=show, args=()).start()


def initialize(c: Config, gui):
    if c.get("first_launch", True, False):
        helper.create_shortcut(gui)
        c.set("first_launch", False)

    try:
        auto_upgrade()
        helper.ping(c, "started")
    except Exception:
        pass

    if not c.get("debug", False):
        The_program_to_hide = win32gui.GetForegroundWindow()
        win32gui.ShowWindow(The_program_to_hide, win32con.SW_HIDE)
        helper.install_thread_excepthook()
        sys.excepthook = helper.unhandled_exception_logging


def wait_and_check():
    time.sleep(10)
    if not G.FishingStarted:
        logging.info("\nDoesn't look like fishing has started\n"
                     "Make sure ProvisionsChalutier addon is visible on top left corner of the screen")


def main():
    c = Config()
    events_buffer = []
    gui = GUI(c, lambda a, b=None: events_buffer.append((a, b)))
    gui.start()
    logging.info(f"Fishybot v{fishy.__version__}")

    initialize(c, gui)

    helper.check_addon()

    bot = Fishy(gui, events_buffer)
    bot.start_event_handler()
    helper.ping(c, f"closed,fishes:{G.totalFishCaught}")


if __name__ == "__main__":
    main()



"""
Defines different fishing modes (states) which acts as state for state machine
also implements callbacks which is called when states are changed
"""
import logging
import time
from abc import abstractmethod, ABC

import pyautogui

from fishy.systems.globals import G
from fishy.systems.helper import round_float
import fishy.systems.fishy_network as net


class FishEvent(ABC):
    @abstractmethod
    def onEnterCallback(self, previousMode):
        pass

    @abstractmethod
    def onExitCallback(self, currentMode):
        pass


class HookEvent(FishEvent):
    def __init__(self, action_key: str, collect_r: bool):
        self.action_key = action_key
        self.collect_r = collect_r


    def onEnterCallback(self, previousMode):
        """
        called when the fish hook is detected
        increases the `fishCaught`  and `totalFishCaught`, calculates the time it took to catch
        presses e to catch the fish

        :param previousMode: previous mode in the state machine
        """
        G.fishCaught += 1
        G.totalFishCaught += 1
        timeToHook = time.time() - G.stickInitTime
        logging.info("HOOOOOOOOOOOOOOOOOOOOOOOK....... " + str(G.fishCaught) + " caught " + "in " + str(
            round_float(timeToHook)) + " secs.  " + "Total: " + str(G.totalFishCaught))
        pyautogui.press(self.action_key)

        if self.collect_r:
            time.sleep(0.1)
            pyautogui.press('r')
            time.sleep(0.1)

    def onExitCallback(self, currentMode):
        pass


class LookEvent(FishEvent):
    """
    state when looking on a fishing hole
    """
    def onEnterCallback(self, previousMode):
        """
        presses e to throw the fishing rod
        :param previousMode: previous mode in the state machine
        """
        pyautogui.press('e')

    def onExitCallback(self, currentMode):
        pass


class IdleEvent(FishEvent):
    """
    State when the fishing hole is depleted or the bot is doing nothing
    """

    def __init__(self, use_net):
        """
        sets the flag to send notification on phone
        :param use_net: true if user wants to send notification on phone
        """
        self.use_net = use_net

    def onEnterCallback(self, previousMode):
        """
        Resets the fishCaught counter and logs a message depending on the previous state
        :param previousMode: previous mode in the state machine
        """

        G.fishCaught = 0
        if self.use_net:
            net.sendHoleDeplete(G.fishCaught)

        if previousMode.name == "hook":
            logging.info("HOLE DEPLETED")
        elif previousMode.name == "stick":
            logging.info("FISHING INTERRUPTED")

    def onExitCallback(self, currentMode):
        pass


class StickEvent(FishEvent):
    """
    State when fishing is going on
    """

    def onEnterCallback(self, previousMode):
        """
        resets the fishing timer
        :param previousMode: previous mode in the state machine
        """
        G.stickInitTime = time.time()
        G.FishingStarted = True

    def onExitCallback(self, currentMode):
        pass
